package com.ruoyi.web.work.controller;

import java.util.Arrays;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.web.work.domain.Banner;
import com.ruoyi.web.work.service.IBannerService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 轮播图
 * @author lsw
 * @date 2024-04-15
 */
@RestController
@RequestMapping("/work/banner")
public class BannerController extends BaseController {
    @Autowired
    private IBannerService bannerService;

    @PreAuthorize("@ss.hasPermi('work:banner:list')")
    @GetMapping("/list")
    public TableDataInfo list(Banner banner){
        startPage();
        List<Banner> list = bannerService.selectList(banner);
        return getDataTable(list);
    }

    @PreAuthorize("@ss.hasPermi('work:banner:query')")
    @GetMapping(value = "/detail/{id}")
    public AjaxResult detail(@PathVariable("id") Long id){
        return AjaxResult.success(bannerService.getById(id));
    }

    @PreAuthorize("@ss.hasPermi('work:banner:add')")
    @Log(title = "轮播图", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public AjaxResult add(@RequestBody Banner banner){
        return toAjax(bannerService.save(banner));
    }

    @PreAuthorize("@ss.hasPermi('work:banner:edit')")
    @Log(title = "轮播图", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody Banner banner){
        return toAjax(bannerService.updateById(banner));
    }

    @PreAuthorize("@ss.hasPermi('work:banner:remove')")
    @Log(title = "轮播图", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids){
        return toAjax(bannerService.removeByIds(Arrays.asList(ids)));
    }
}
